/*
 *  Licensed to the Apache Software Foundation (ASF) under one
 *  or more contributor license agreements.  See the NOTICE file
 *  distributed with this work for additional information
 *  regarding copyright ownership.  The ASF licenses this file
 *  to you under the Apache License, Version 2.0 (the
 *  "License"); you may not use this file except in compliance
 *  with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an
 *   * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 *  KIND, either express or implied.  See the License for the
 *  specific language governing permissions and limitations
 *  under the License.
 */

package org.wso2.registry.servlet;

import org.wso2.registry.*;
import org.wso2.registry.secure.RegistryUserManager;
import org.wso2.registry.secure.SecureRegistry;
import org.wso2.usermanager.Realm;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;

// todo: editing UI is not a part of final registry UI. so remove this before the final release.
public class EditProcessor {

    public static final String STATUS_MESSAGE_NAME = "edit_status";

    private String contextRoot = null;

    private void initContextRoot(HttpServletRequest req) {
        if (contextRoot != null && contextRoot.trim().length() != 0) {
            return;
        }
        String contextPath = req.getContextPath();
        //handling ROOT scenario, for servlets in the default (root) context, this method returns ""
        if (contextPath != null && contextPath.length() == 0) {
            contextPath = "/";
        }
        this.contextRoot = contextPath;
    }

    /**
     * Get an absolute path from a current path and a target path.  If
     * the target path begins with a "/" it's absolute, so just use it.
     * Otherwise it's a relative path from currentPath.
     *
     * @param currentPath our current registry path
     * @param path the target path (may be relative or absolute)
     * @return an absolute path
     */
    private String getAbsolutePath(String currentPath, String path) {
        if (path == null || path.length() == 0) return null;
        if (path.charAt(0) == '/') return path;
        if (currentPath == null || "/".equals(currentPath)) return "/" + path;
        return currentPath + "/" + path;
    }

    public boolean processEdit(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {

        Registry registry;
        try {
            registry = Utils.getSecureRegistry(request);
        } catch (RegistryException e) {
            e.printStackTrace();
            request.getSession().setAttribute(
                    EditProcessor.STATUS_MESSAGE_NAME, e.getMessage());
            return false;
        }

        initContextRoot(request);

        // check if this is for editing resources
        String uri = request.getRequestURI();
        String p1 = uri.substring(contextRoot.length(), uri.length());

        if (p1.startsWith("/edit")) {

            // Get the correct path
            String currentPath = (String)request.getSession().getAttribute("path");
            String path = request.getParameter("path");
            path = getAbsolutePath(currentPath, path);

            if (p1.startsWith("/")) {
                p1 = p1.substring(1, p1.length());
            }

            if (p1.endsWith("/")) {
                p1 = p1.substring(0, p1.length() - 1);
            }

            String[] parts = p1.split("/");
            if (parts != null && parts.length > 0) {
                if (parts[0].equalsIgnoreCase("edit")) {

                    if (parts.length == 1) {
                        request.getSession().setAttribute(EditProcessor.STATUS_MESSAGE_NAME,
                                "Ok");

                        response.setContentType("text/html");
                        request.getRequestDispatcher("/admin/edit_resource.jsp")
                                .include(request, response);
                        return true;

                    } else if (parts.length == 2) {

                        if (parts[1].equals("tag")) {

                            String tag = request.getParameter("tag");

                            try {
                                registry.applyTag(path, tag);
                                request.getSession().setAttribute(
                                        EditProcessor.STATUS_MESSAGE_NAME, "Tag " + tag +
                                        " is successfully applied to the resource " + path);
                            } catch (RegistryException e) {
                                e.printStackTrace();
                                request.getSession().setAttribute(
                                        EditProcessor.STATUS_MESSAGE_NAME, e.getMessage());
                            }

                        } else if (parts[1].equals("comment")) {

                            String comment = request.getParameter("comment");

                            try {
                                registry.addComment(path, new Comment(comment));
                                request.getSession().setAttribute(
                                        EditProcessor.STATUS_MESSAGE_NAME, "Comment \"" + comment +
                                        "\" is successfully added to the resource " + path);
                            } catch (RegistryException e) {
                                e.printStackTrace();
                                request.getSession().setAttribute(
                                        EditProcessor.STATUS_MESSAGE_NAME, e.getMessage());
                            }

                        } else if (parts[1].equals("rate")) {

                            int rating = Integer.parseInt(request.getParameter("rating"));

                            try {
                                registry.rateResource(path, rating);
                                request.getSession().setAttribute(
                                        EditProcessor.STATUS_MESSAGE_NAME, "Resource " + path +
                                        " is successfully rated with the rating " + rating);
                            } catch (RegistryException e) {
                                e.printStackTrace();
                                request.getSession().setAttribute(
                                        EditProcessor.STATUS_MESSAGE_NAME, e.getMessage());
                            }

                        } else if (parts[1].equals("delete")) {

                            try {
                                registry.delete(path);
                                request.getSession().setAttribute(
                                        EditProcessor.STATUS_MESSAGE_NAME, "Resource " + path +
                                        " deleted successfully");
                            } catch (RegistryException e) {
                                e.printStackTrace();
                                request.getSession().setAttribute(
                                        EditProcessor.STATUS_MESSAGE_NAME, e.getMessage());
                            }

                        } else if (parts[1].equals("import")) {

                            String fromURI = request.getParameter("fromURI");
                            String toPath = request.getParameter("toPath");

                            try {
                                ResourceImporter resourceImporter = new ResourceImporter(registry);
                                resourceImporter.importDirectory(fromURI, toPath);
                                request.getSession().setAttribute(
                                        EditProcessor.STATUS_MESSAGE_NAME, "Resource " + fromURI +
                                        " is imported successfully to the path " + toPath + ".");
                            } catch (RegistryException e) {
                                e.printStackTrace();
                                request.getSession().setAttribute(
                                        EditProcessor.STATUS_MESSAGE_NAME, e.getMessage());
                            }

                        } else if (parts[1].equals("addCollection")) {

                            String type = request.getParameter("type");
                            String description = request.getParameter("description");

                            if (type.equalsIgnoreCase("collection")) {

                                try {
                                    Resource collection = new Resource();
                                    collection.setDirectory(true);
                                    collection.setDescription(description);
                                    registry.put(path, collection);
                                    request.getSession().setAttribute(
                                            EditProcessor.STATUS_MESSAGE_NAME, "Collection " +
                                            path + " is successfully created.");
                                } catch (RegistryException e) {
                                    e.printStackTrace();
                                    request.getSession().setAttribute(
                                            EditProcessor.STATUS_MESSAGE_NAME, e.getMessage());
                                }
                            }
                        } else if (parts[1].equals("signin")) {

                            try {
                                Registry coreRegistry = (Registry) request.getSession().
                                        getServletContext().getAttribute(RegistryConstants.REGISTRY);

                                Realm realm = (Realm) request.getSession().getServletContext().
                                        getAttribute(RegistryConstants.REGISTRY_REALM);

                                String userName = request.getParameter("userName");
                                String password = request.getParameter("password");
                                SecureRegistry secureRegistry =
                                        new SecureRegistry(userName, password, coreRegistry);
                                Utils.setSecureRegistry(request, secureRegistry);

                                request.getSession().setAttribute(EditProcessor.STATUS_MESSAGE_NAME,
                                        "Signed in sucessfully as " + userName);
                            } catch (RegistryException e) {
                                e.printStackTrace();
                                request.getSession().setAttribute(
                                        EditProcessor.STATUS_MESSAGE_NAME, e.getMessage());
                            }

                        } else if (parts[1].equals("signout")) {

                            try {
                                Registry coreRegistry = (Registry) request.getSession().
                                        getServletContext().getAttribute(RegistryConstants.REGISTRY);

                                Realm realm = (Realm) request.getSession().getServletContext().
                                        getAttribute(RegistryConstants.REGISTRY_REALM);

//                                String userName = request.getParameter("userName");
                                SecureRegistry secureRegistry = new SecureRegistry(
                                        RegistryConstants.ANONYMOUS_USER, "guest", coreRegistry);
                                Utils.setSecureRegistry(request, secureRegistry);

                                request.getSession().setAttribute(EditProcessor.STATUS_MESSAGE_NAME,
                                        "Signed out sucessfully");

                            } catch (RegistryException e) {
                                e.printStackTrace();
                                request.getSession().setAttribute(
                                        EditProcessor.STATUS_MESSAGE_NAME, e.getMessage());
                            }

                        } else if (parts[1].equals("addUser")) {

                            String userName = request.getParameter("userName");
                            String password = request.getParameter("password");

                            try {
                                SecureRegistry secureRegistry = (SecureRegistry)registry;
                                RegistryUserManager registryUserManager =
                                        secureRegistry.getUserManager();
                                registryUserManager.addUser(userName, password);

                                request.getSession().setAttribute(EditProcessor.STATUS_MESSAGE_NAME,
                                        "User added sucessfully");
                            } catch (RegistryException e) {
                                e.printStackTrace();
                                    request.getSession().setAttribute(
                                            EditProcessor.STATUS_MESSAGE_NAME, e.getMessage());
                            }

                        } else if (parts[1].equals("grant")) {

                            String userName = request.getParameter("userName");
                            String actionParam = request.getParameter("action");
                            String action = "";
                            if (actionParam.equalsIgnoreCase("get")) {
                                action = ActionConstants.GET;
                            } else if (actionParam.equalsIgnoreCase("put")) {
                                action = ActionConstants.PUT;
                            } else if (actionParam.equalsIgnoreCase("delete")) {
                                action = ActionConstants.DELETE;
                            }

                            try {
                                SecureRegistry secureRegistry = (SecureRegistry)registry;
                                RegistryUserManager registryUserManager =
                                        secureRegistry.getUserManager();
                                registryUserManager.authorizeUser(userName, path, action);

                                request.getSession().setAttribute(EditProcessor.STATUS_MESSAGE_NAME,
                                        "Permission granted sucessfully");
                            } catch (RegistryException e) {
                                e.printStackTrace();
                                    request.getSession().setAttribute(
                                            EditProcessor.STATUS_MESSAGE_NAME, e.getMessage());
                            }

                        } else if (parts[1].equals("deny")) {

                            String userName = request.getParameter("userName");
                            String actionParam = request.getParameter("action");
                            String action = "";
                            if (actionParam.equalsIgnoreCase("get")) {
                                action = ActionConstants.GET;
                            } else if (actionParam.equalsIgnoreCase("put")) {
                                action = ActionConstants.PUT;
                            } else if (actionParam.equalsIgnoreCase("delete")) {
                                action = ActionConstants.DELETE;
                            }

                            try {
                                SecureRegistry secureRegistry = (SecureRegistry)registry;
                                RegistryUserManager registryUserManager =
                                        secureRegistry.getUserManager();
                                registryUserManager.denyUser(userName, path, action);
                                
                                request.getSession().setAttribute(EditProcessor.STATUS_MESSAGE_NAME,
                                        "Permission denied sucessfully");
                            } catch (RegistryException e) {
                                e.printStackTrace();
                                    request.getSession().setAttribute(
                                            EditProcessor.STATUS_MESSAGE_NAME, e.getMessage());
                            }
                        }

                        response.setContentType("text/html");
                        request.getRequestDispatcher(
                                "/admin/edit_resource.jsp").forward(request, response);
                        return true;
                    }
                }
            }
        }

        return false;
    }
}
